@@ -35,7 +35,7 @@
 #include <gconf/gconf-client.h>
 
 #include "acme.h"
-#include "volume-oss.h"
+#include "g-volume.h"
 #ifdef USE_FBLEVEL
 #include "fb-level.h"
 #endif
@@ -44,7 +44,7 @@ static gboolean event_cb (GtkWidget * widget, GdkEvent * event,
 			  gpointer data);
 
 typedef struct {
-	VolumeOSS *volobj;
+	GVolume *volobj;
 #ifdef USE_FBLEVEL
 	FBLevel *levobj;
 #endif
@@ -65,55 +65,55 @@ enum {
 };
 
 static void
+selection_get_func (GtkClipboard *clipboard, GtkSelectionData *selection_data,
+		guint info, gpointer user_data_or_owner)
+{
+}
+
+static void
+selection_clear_func (GtkClipboard *clipboard, gpointer user_data_or_owner)
+{       
+	return;
+}
+
+#define SELECTION_NAME "_ACME_SELECTION"
+
+static gboolean
 acme_get_lock (Acme *acme)
 {
-	char *path;
-	int fd;
+	gboolean result = FALSE;
+	GtkClipboard *clipboard;
+	Atom clipboard_atom = gdk_x11_get_xatom_by_name (SELECTION_NAME);
+	static const GtkTargetEntry targets[] = {
+		{ SELECTION_NAME, 0, 0 }
+	};
 
-	acme->lock_fd = -1;
+	XGrabServer (GDK_DISPLAY());
 
-	path = g_strdup_printf ("%s/.acme.lock", g_get_home_dir());
-	fd = open (path, O_WRONLY | O_CREAT, S_IRWXU);
-#ifdef DEBUG
-	g_message ("opened %s, got fd %d", path, fd);
-#endif
+	if (XGetSelectionOwner (GDK_DISPLAY(), clipboard_atom) != None)
+		goto out;
 
-	g_free (path);
-	if (fd < 0)
-	{
-#ifdef DEBUG
-		g_message ("Couldn't open the lock file");
-#endif
-		return;
-	}
+	clipboard = gtk_clipboard_get (gdk_atom_intern (SELECTION_NAME, FALSE));
 
-	if (lockf (fd, F_TEST, 0L) < 0)
-	{
-#ifdef DEBUG
-		g_message ("Couldn't get lock on file");
-#endif
-		close (fd);
-		return;
-	}
+	if (!gtk_clipboard_set_with_data  (clipboard, targets,
+				G_N_ELEMENTS (targets),
+				selection_get_func,
+				selection_clear_func, NULL))
+		goto out;
 
-	if (lockf (fd, F_TLOCK, 0L) < 0)
-	{
-#ifdef DEBUG
-		g_message ("for some reason, we couldn't get the lock");
-#endif
-		close (fd);
-		return;
-	}
+	result = TRUE;
 
-	acme->lock_fd = fd;
-	return;
+out:
+	XUngrabServer (GDK_DISPLAY());
+	gdk_flush();
+
+	return result;
 }
 
 static void
 acme_exit (Acme *acme)
 {
-	g_object_unref (acme->conf_client);
-	if (acme->lock_fd >= 0)
+	if (acme->lock_fd > 0)
 	{
 		char *path;
 
@@ -141,6 +141,7 @@ acme_error (char * msg)
 			GTK_RESPONSE_OK);
 	gtk_widget_show (error_dialog);
 	gtk_dialog_run (GTK_DIALOG (error_dialog));
+	gtk_widget_destroy (error_dialog);
 }
 
 static void
@@ -157,8 +158,6 @@ execute (char * cmd)
 
 		acme_error (msg);
 		g_free (msg);
-
-		exit (1);
 	}
 }
 
@@ -190,23 +189,37 @@ acme_play_sound (Acme *acme)
 	g_free (command);
 }
 
-static gboolean
-vol_problem_cb (VolumeOSS *fb, gpointer data)
+static char*
+permission_problem_string (const char *files)
 {
-	acme_error(_("Permissions on the file /dev/mixer are broken\n"
-		"Please check ACME's documentation, correct the problem "
-		"and restart ACME."));
+	return g_strdup_printf (_("Permissions on the file %s are broken\n"
+				"Please check ACME's documentation, correct "
+				"the problem and restart ACME."), files);
+}
+
+static gboolean
+vol_problem_cb (GVolume *fb, gpointer data)
+{
+	char *msg;
+
+	msg = permission_problem_string ("/dev/mixer");
+	acme_error (msg);
+	g_free (msg);
+
 	return TRUE;
 }
 
 #ifdef USE_FBLEVEL
-static gboolean
-fb_problem_cb (FBLevel *fb, gpointer data)
+static void
+fb_problem_cb (void)
 {
-	acme_error(_("Permissions on the files /dev/fb0 and/or /dev/pmu "
-		"are broken.\nPlease check ACME's documentation, correct "
-		"the problem and restart ACME."));
-	return TRUE;
+	char *msg;
+
+	msg = permission_problem_string ("/dev/pmu");
+	acme_error (msg);
+	g_free (msg);
+
+	return;
 }
 #endif
 
@@ -220,7 +233,7 @@ update_use_pcm_cb (GConfClient *client, guint id, GConfEntry *entry,
 	use_pcm = gconf_client_get_bool (acme->conf_client,
 			"/apps/acme/use_pcm",
 			NULL);
-	volume_oss_set_use_pcm (acme->volobj, use_pcm);
+	g_volume_set_use_pcm (acme->volobj, use_pcm);
 }
 
 static void
@@ -342,6 +355,9 @@ init_kbd (Acme *acme)
 		tmp = gconf_client_get_int (acme->conf_client,
 				keys[i].key_config,
 				NULL);
+
+		keys[i].key_code = tmp;
+
 		if (tmp != -1)
 		{
 #ifdef DEBUG
@@ -351,7 +367,6 @@ init_kbd (Acme *acme)
 #endif
 			grab_key (keys [i].key_code);
 		}
-		keys[i].key_code = tmp;
 	}
 
 	for (i = 0; i < HANDLED_KEYS; i++) {
@@ -514,33 +529,33 @@ do_sound_action (Acme *acme, int type)
 		acme->dialog_timeout = 0;
 	}
 
-	vol = volume_oss_get_volume (acme->volobj);
-	muted = volume_oss_get_mute (acme->volobj);
+	vol = g_volume_get_volume (acme->volobj);
+	muted = g_volume_get_mute (acme->volobj);
 
 	switch (type) {
 	case MUTE_KEY:
-		volume_oss_mute_toggle(acme->volobj);
+		g_volume_mute_toggle(acme->volobj);
 		break;
 	case VOLUME_DOWN_KEY:
 		if (muted) {
-			volume_oss_mute_toggle(acme->volobj);
+			g_volume_mute_toggle(acme->volobj);
 		} else {
-			volume_oss_set_volume (acme->volobj, vol - VOLUME_STEP);
+			g_volume_set_volume (acme->volobj, vol - VOLUME_STEP);
 		}
 		break;
 	case VOLUME_UP_KEY:
 		if (muted) {
-			volume_oss_mute_toggle(acme->volobj);
+			g_volume_mute_toggle(acme->volobj);
 		} else {
-			volume_oss_set_volume (acme->volobj, vol + VOLUME_STEP);
+			g_volume_set_volume (acme->volobj, vol + VOLUME_STEP);
 		}
 		break;
 	}
 
-	muted = volume_oss_get_mute(acme->volobj);
+	muted = g_volume_get_mute(acme->volobj);
 	acme_image_set (acme, muted ? ICON_MUTED : ICON_LOUD);
 
-	vol = volume_oss_get_volume (acme->volobj);
+	vol = g_volume_get_volume (acme->volobj);
 	progress = glade_xml_get_widget (acme->xml, "progressbar");
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
 			(double) vol / 100);
@@ -573,7 +588,8 @@ do_action (int type, Acme *acme)
 #ifdef USE_FBLEVEL
 	case BRIGHT_DOWN_KEY:
 	case BRIGHT_UP_KEY:
-
+		if (acme->levobj == NULL)
+			break;
 		do_brightness_action (acme, type);
 		break;
 #endif
@@ -651,11 +667,14 @@ main (int argc, char *argv[])
 			argc, argv,
 			NULL);
 
-	acme = g_new (Acme, 1);
+	acme = g_new0 (Acme, 1);
+	acme->conf_client = NULL;
 
-	acme_get_lock (acme);
-	if (acme->lock_fd < 0)
+	if (acme_get_lock (acme) == FALSE)
+	{
+		g_print ("Acme is already running, exiting...\n");
 		acme_exit (acme);
+	}
 
 	glade_gnome_init ();
 	acme->xml = glade_xml_new (ACME_DATA "acme.glade", NULL, NULL);
@@ -682,11 +701,19 @@ main (int argc, char *argv[])
 	acme->dialog_timeout = 0;
 
 	/* initialise Volume handler */
-	acme->volobj = volume_oss_new();
+	acme->volobj = g_volume_new();
+	if (acme->volobj == NULL)
+	{
+		acme_error (_("Your system has no sound support, or ACME was"
+				" compiled without any sound support\n"
+				"ACME will now exit"));
+		exit (1);
+	}
 	g_signal_connect(G_OBJECT(acme->volobj),
 			"fd_problem",
 			(GCallback) vol_problem_cb, NULL);
-	volume_oss_set_use_pcm (acme->volobj,
+
+	g_volume_set_use_pcm (acme->volobj,
 			gconf_client_get_bool (acme->conf_client,
 				"/apps/acme/use_pcm", NULL));
 	gconf_client_notify_add (acme->conf_client,
@@ -697,9 +724,8 @@ main (int argc, char *argv[])
 #ifdef USE_FBLEVEL
 	/* initialise Frame Buffer level handler */
 	acme->levobj = fb_level_new();
-	g_signal_connect(G_OBJECT(acme->levobj),
-			"fd_problem",
-			(GCallback) fb_problem_cb, NULL);
+	if (acme->levobj == NULL)
+		fb_problem_cb ();
 #endif
 
 	g_signal_connect (GTK_OBJECT (acme->invisible),
